#
# most-fishing.R, 26 Jun 22
#
# Data from:
# Are most and more than half truth-conditionally equivalent?
# Milica Denić and Jakub Szymanik
#
# Code adapted from Most-MTH-functions.R and Experiment1.R
#

library("colorspace")
library(plyr)


par(bty="l")
par(las=1)
par(pch="+") 
pal_col=rainbow(2)


# Remove non-native speakers of English, even if they cannot spell
non_native_excl = function(df)
{
df$NativeEnglish=toupper(df$language) %in% c("ENGLISH", "ENG", "EN", "ENGLISH ", "UNITED STATES",
						"ENLISH", "ENGLISH (US)", "ENLGISH", "UNITED KINGDOM",
						"BRITISH", "EMGLISH", " ENGLISH", "ENGLIS", "CANADA",
						"ENLGLISH", "NGLISH", "EN_US", "ENGLISH LANGUAGE ")
df=subset(df, NativeEnglish)
return(df)
}


# Remove based on performance on uncontroversial controls
control_excl = function(d)
{
###Exclude based on responses in the control conditions
#For positive sentences, we expect people to say true with 'some' at least for 0 < X < 50 
excl1=subset(d, ((Quantifier == "Some" & Percentage < 50)|(Quantifier == "All")))
excl1=mutate(excl1, expected = ifelse((Quantifier == "Some" & Percentage > 0)|(Quantifier == "All" & Percentage == 100), 1, 0))
excl1=mutate(excl1, correct = ifelse(excl1$answernum == excl1$expected, 1, 0))

#For negative sentences, we expect people to say true with ' not all' at least for 50 > X > 100 
excl2=subset(d, ((Quantifier == "All-Neg" & Percentage > 50)|(Quantifier == "Some-Neg")))
excl2=mutate(excl2, expected = ifelse((Quantifier == "Some-Neg" & Percentage == 0) |
					(Quantifier == "All-Neg" & Percentage < 100), 1, 0))
excl2=mutate(excl2, correct = ifelse(excl2$answernum == excl2$expected, 1, 0))

excl=rbind(excl1, excl2)

# Calculate for each participant his score on controls
excl.score=ddply(excl, c("ProlificID"), 
                      function(df)c(CTLmean=mean(df$correct)))

# Keep only the participants who scored at least 80% on controls
d=subset(d, ProlificID %in% subset(excl.score, excl.score$CTLmean>= 0.8)$ProlificID)
return(d)
}

### Exclude responses whose RT takes more/less than 2 SD from the mean logRT within each experiment
RT_excl = function(d)
{
d$logRT <-log(d$RT) #Take the log from response time.
  dRT = d %>% group_by(Exp) %>%  summarise(
    RTbound1 = mean(logRT)+sd(logRT)*c(-2),
    RTbound2 = mean(logRT)+sd(logRT)*c(2)
  )

d = merge(d, dRT, by = c("Exp"))
d=subset(d, logRT>RTbound1 & logRT<RTbound2)
return(d)
}

# Function for upward versions, start scale = 4 (simple logistic function)
model1=function(x)
{
try(coef(nls(answernum ~ 1/(1+exp((xmid-Percentage)/scal)), data=x, 
				start=list(xmid=50, scal=4))))
}

# Function for downward versions, start scale = -4 (simple logistic function)
model2=function(x)
{
try(coef(nls(answernum ~ 1/(1+exp((Percentage-xmid)/scal)), data=x, 
				start=list(xmid=50, scal=4))))
}


# Plot Percentages/Answer for an individual
indiv_perc=function(pid_num)
{
pid1=subset(M1, ProlificID == uPID[pid_num])
plot(pid1$Percentage, pid1$answernum)
}


# Use changepoint analysis to find switch-over point
change_pt=function(df)
{
p_order=df[order(df$Percentage), ]
# Data is binary, doe snot have a normal distribution
pt=cpt.mean(p_order$answernum, penalty="Manual", test.stat="CUSUM", Q=1)
# Check for no change-point detected
if ((pt@cpts[1] == nrow(df)) | (pt@cpts[1] == 1))
   return(NA)
mid_pt=(p_order$Percentage[pt@cpts[1]]+p_order$Percentage[pt@cpts[1]-1])/2
# print(c(mid_pt, df$ProlificID[1]))

return(mid_pt)
}



Folder = "./"

# This is the experiment-one data, post 'load.data' and removal of
# columns containing a single value.
d=read.csv("exp1.csv", as.is=TRUE)

d$answernum=ifelse(d$answer == "True", 1, 0)

# How many unique subjects?
# length(unique(d$ProlificID))

# Load the design files
design1=read.csv(paste0(Folder, "Experiment1-most-design.csv"), as.is=TRUE)
design2=read.csv(paste0(Folder, "Experiment1-mth-design.csv"), as.is=TRUE)

# Merge the results with the design file
d1 = merge(subset(d, (Exp == 1| Exp == 2)), design1, by="Rtype")
d2 = merge(subset(d, (Exp == 3| Exp == 4)), design2, by="Rtype")
d = rbind(d1, d2)

### Exclude non-native speakers of English
d = non_native_excl(d)

###Exclude based on performance in the control conditions
d = control_excl(d)

### Exclude responses whose RT takes more/less than 2 SD from the mean logRT within each experiment
# d = RT_excl(d)

# Plot by individual
uPID=unique(d$ProlificID)

M1=subset(d, Exp == 1 & Quantifier == "Most")
uPID=unique(M1$ProlificID)
indiv_perc(1)

# # fit simple logistic function
# A logistic fit fails on the data from around 40% of subjects
# fitted1=dlply(subset(d, Exp == 1 & Quantifier == "Most"), .(ProlificID), model1)
# fitted1success=fitted1[(which(sapply(fitted1,is.numeric),arr.ind=TRUE))]

# Use change-point analysis, which works when there is one (a few subjects don't)
cpt_1=ddply(subset(d, Exp == 1 & Quantifier == "Most"), .(ProlificID), change_pt)

# fitted2=dlply(subset(d, Exp == 2 & Quantifier == "Most"), .(ProlificID), model2)
# fitted2success=fitted2[(which(sapply(fitted2,is.numeric), arr.ind=TRUE))]
cpt_2=ddply(subset(d, Exp == 2 & Quantifier == "Most"), .(ProlificID), change_pt)

# fitted3=dlply(subset(d, Exp == 3 & Quantifier == "More-than-half"), .(ProlificID), model1)
# fitted3success=fitted3[(which(sapply(fitted3,is.numeric),arr.ind=TRUE))]
cpt_3=ddply(subset(d, Exp == 3 & Quantifier == "More-than-half"), .(ProlificID), change_pt)

# fitted4=dlply(subset(d, Exp == 4 & Quantifier == "More-than-half"), .(ProlificID), model2)
# fitted4success=fitted4[(which(sapply(fitted4,is.numeric), arr.ind=TRUE))]
cpt_4=ddply(subset(d, Exp == 4 & Quantifier == "More-than-half"), .(ProlificID), change_pt)

pal_col=rainbow(4)

plot(count(cpt_1$V1), type="b", log="y", col=pal_col[1],
	xlim=c(35, 75), ylim=c(1, 50),
	xlab="Percentage", ylab="Subjects")
points(count(cpt_2$V1), type="b", col=pal_col[2])
points(count(cpt_3$V1), type="b", col=pal_col[3])
points(count(cpt_4$V1), type="b", col=pal_col[4])

legend("topright", legend=c("Most upward", "Most downward", "More than half upward", "More than half downward"),
			fill=pal_col, border="white", bty="n")


